@@ -8,15 +8,18 @@
  * file.
  *
  * Copyright (c) 1992 by I. Lee Hetherington, all rights reserved.
+ * Copyright (c) 1998-2013 Eddie Kohler
  *
- * Permission is hereby granted to use, modify, and distribute this program
- * for any purpose provided this copyright notice and the one below remain
- * intact.
- *
- * I. Lee Hetherington (ilh@lcs.mit.edu)
- *
- * 1.5 and later versions contain changes by, and are maintained by,
- * Eddie Kohler <ekohler@gmail.com>.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, subject to the
+ * conditions listed in the Click LICENSE file, which is available in full at
+ * http://github.com/kohler/click/blob/master/LICENSE. The conditions
+ * include: you must preserve this copyright notice, and you cannot mention
+ * the copyright holders in advertising related to the Software without
+ * their permission. The Software is provided WITHOUT ANY WARRANTY, EXPRESS
+ * OR IMPLIED. This notice is a summary of the Click LICENSE file; the
+ * license in that file is binding.
  *
  * New change log in `NEWS'. Old change log:
  *
@@ -92,8 +95,8 @@ static int lenIV = 4;
 static char cs_start[10];
 
 /* for charstring buffering */
-static byte charstring_buf[65535];
-static byte *charstring_bp;
+static byte *charstring_buf, *charstring_bp;
+static int charstring_bufsiz;
 
 /* decryption stuff */
 static uint16_t er, cr;
@@ -109,6 +112,7 @@ static struct command {
   { "and", 12, 3 },		/* Type 2 */
   { "blend", 16, -1 },		/* Type 2 */
   { "callgsubr", 29, -1 },	/* Type 2 */
+  { "callother", 12, 16 },	/* Type 1 ONLY */
   { "callothersubr", 12, 16 },	/* Type 1 ONLY */
   { "callsubr", 10, -1 },
   { "closepath", 9, -1 },	/* Type 1 ONLY */
@@ -371,6 +375,12 @@ static void charstring_start(void)
 {
   int i;
 
+  if (!charstring_buf) {
+    charstring_bufsiz = 65536;
+    if (!(charstring_buf = (byte *) malloc(charstring_bufsiz)))
+      fatal_error("out of memory");
+  }
+
   charstring_bp = charstring_buf;
   cr = 4330;
   for (i = 0; i < lenIV; i++)
@@ -381,10 +391,14 @@ static void charstring_start(void)
 
 static void charstring_byte(int v)
 {
-    byte b = (byte)(v & 0xff);
-    if (charstring_bp - charstring_buf > (int)sizeof(charstring_buf))
-	fatal_error("charstring buffer overflow");
-    *charstring_bp++ = cencrypt(b);
+  byte b = (byte)(v & 0xff);
+  if (charstring_bp - charstring_buf == charstring_bufsiz) {
+    charstring_bufsiz *= 2;
+    if (!(charstring_buf = (byte *) realloc(charstring_buf, charstring_bufsiz)))
+      fatal_error("out of memory");
+    charstring_bp = charstring_buf + charstring_bufsiz / 2;
+  }
+  *charstring_bp++ = cencrypt(b);
 }
 
 /* This function outputs buffered, encrypted charstring data through possible
