@@ -8,15 +8,18 @@
  * PFA format.
  *
  * Copyright (c) 1992 by I. Lee Hetherington, all rights reserved.
+ * Copyright (c) 1998-2013 Eddie Kohler
  *
- * Permission is hereby granted to use, modify, and distribute this program
- * for any purpose provided this copyright notice and the one below remain
- * intact.
- *
- * I. Lee Hetherington (ilh@lcs.mit.edu)
- *
- * 1.5 and later versions contain changes by, and are maintained by,
- * Eddie Kohler <ekohler@gmail.com>.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, subject to the
+ * conditions listed in the Click LICENSE file, which is available in full at
+ * http://github.com/kohler/click/blob/master/LICENSE. The conditions
+ * include: you must preserve this copyright notice, and you cannot mention
+ * the copyright holders in advertising related to the Software without
+ * their permission. The Software is provided WITHOUT ANY WARRANTY, EXPRESS
+ * OR IMPLIED. This notice is a summary of the Click LICENSE file; the
+ * license in that file is binding.
  *
  * New change log in `NEWS'. Old change log:
  *
@@ -64,6 +67,7 @@
 #include <limits.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <assert.h>
 #include <lcdf/clp.h>
 #include "t1lib.h"
 
@@ -314,6 +318,11 @@ static int save_cap = 0;
 static void
 append_save(const unsigned char *line, int len)
 {
+  if (line == save) {
+    assert(len <= save_cap);
+    save_len = len;
+    return;
+  }
   if (save_len + len >= save_cap) {
     unsigned char *new_save;
     if (!save_cap) save_cap = 1024;
@@ -363,6 +372,7 @@ eexec_line(unsigned char *line, int line_len)
 	append_save(line, line_len);
 	line = save;
 	line_len = save_len;
+        save_len = 0;
     }
 
     if (!line_len)
@@ -416,12 +426,10 @@ eexec_line(unsigned char *line, int line_len)
 	    decrypt_charstring(line + pos + 2 + cs_start_len, cs_len);
 	    pos += 2 + cs_start_len + cs_len;
 	    fprintf(ofp, "\t}%.*s", line_len - pos, line + pos);
-	    save_len = 0;
 	    return cut_newline;
 	} else {
 	    /* not long enough! */
-	    if (line != save)
-		append_save(line, line_len);
+            append_save(line, line_len);
 	    return 0;
 	}
     }
@@ -452,7 +460,6 @@ eexec_line(unsigned char *line, int line_len)
     set_lenIV((char *)line);
     set_cs_start((char *)line);
     fprintf(ofp, "%.*s", line_len, line);
-    save_len = 0;
 
     /* look for `currentfile closefile' to see if we should stop decrypting */
     if (oog_memstr(line, line_len, "currentfile closefile", 21) != 0)
